
#ifndef DS1620_H
#define DS1620_H

//============================================================================//

#include <inttypes.h>

#include <avr/io.h>
#include <util/delay.h>

//============================================================================//

#define DS1620_PORT PORTB
#define DS1620_DDR  DDRB
#define DS1620_PIN  PINB

#define DS1620_DQ  PB3
#define DS1620_CLK PB1
#define DS1620_RST PB2

//============================================================================//

void ds1620_init(void);

static void     ds1620_write(uint16_t value, uint8_t number_bits);
static uint16_t ds1620_read (uint8_t number_bits);

void ds1620_write_config(uint8_t config_register);
void ds1620_write_thigh (uint16_t t_high);
void ds1620_write_tlow  (uint16_t t_low);

uint8_t  ds1620_read_config(void);
uint16_t ds1620_read_thigh (void);
uint16_t ds1620_read_tlow  (void);
uint16_t ds1620_read_temp  (void);

void ds1620_start(void);
void ds1620_stop(void);

float ds1620_convert_temp(uint16_t temperature);

//============================================================================//

void ds1620_init(void)
{
	// Data, Clock und Reset als ausgänge schalten.
	DS1620_DDR |= (1 << DS1620_DQ) | (1 << DS1620_CLK) | (1 << DS1620_RST);

	// Data = 0; Clock = 1; Reset = 0;
	DS1620_PORT |= (0 << DS1620_DQ) | (1 << DS1620_CLK) | (0 << DS1620_RST);
}

//============================================================================//

static void ds1620_write(uint16_t value, uint8_t number_bits)
{
	uint8_t i;
	
	_delay_us(1);
	
	// Bits übertragen.
	for(i = 0; i < number_bits; i++)
	{
		// Clock auf 0 setzten.
		DS1620_PORT &= ~(1 << DS1620_CLK);
		
		// Nächstes bit herausfiltern und an die position des data pins 
		// verschieben übertragen.
		if((value & 0x01) == 1)
			DS1620_PORT |= (1 << DS1620_DQ);
		else
			DS1620_PORT &= ~(1 << DS1620_DQ);
		
		_delay_us(1);
		
		// Clock auf 1 setzten und bit damit bestätigen.
		DS1620_PORT |= (1 << DS1620_CLK);
		
		_delay_us(1);
		
		// Bits um eins nach rechts shiften damit beim nächsten mal das 
		// nächste bit an der richtigen stelle steht.
		value = (value >> 1);
	}
}

//============================================================================//

static uint16_t ds1620_read(uint8_t number_bits)
{
	uint8_t i;
	
	// Die gelesenen bits.
	uint16_t result = 0x00;
	
	// Data-Pin als eingang schalten.
	DS1620_DDR &= ~(1 << DS1620_DQ);
	
	// Data-Pin pull-up deaktivieren.
	DS1620_PORT &= ~(1 << DS1620_DQ);
	
	_delay_us(1);
	
	// Bits lesen.
	for(i = 0; i < number_bits; i++)
	{
		// Clock auf 0 setzten.
		DS1620_PORT &= ~(1 << DS1620_CLK);
		
		// NEU
		_delay_us(1);
		
		// Wenn auf dem data-pin 1 liegt.
		if((DS1620_PIN & (1 << DS1620_DQ)) > 0)
		{
			// Ins ergebnis übernehmen.
			result |= (1 << number_bits);
		}
		
		// Alle bits in ergebnis einen nach rechts shiften damit das nächste 
		// bit wieder an die richtige stellt kommt.
		result = (result >> 1);
		
		_delay_us(1);
		
		// Clock auf 1 setzten und bit damit bestätigen.
		DS1620_PORT |= (1 << DS1620_CLK);
		
		_delay_us(1);
	}
	
	// Data-Pin wieder als ausgang schalten.
	DS1620_DDR |= (1 << DS1620_DQ);
	
	return result;
}

//============================================================================//

void ds1620_write_config(uint8_t config_register)
{
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Mit dem nächsten Byte wird das config register überschrieben.
	ds1620_write(0x0C, 8);
	ds1620_write(config_register, 8);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
}

//============================================================================//

void ds1620_write_thigh(uint16_t t_high)
{
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Mit dem nächsten wert wird das t-high register überschrieben.
	ds1620_write(0x01, 8);
	ds1620_write(t_high, 9);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
}

//============================================================================//

void ds1620_write_tlow(uint16_t t_low)
{
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Mit dem nächsten wert wird das t-low register überschrieben.
	ds1620_write(0x02, 8);
	ds1620_write(t_low, 9);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
}

//============================================================================//

uint8_t ds1620_read_config(void)
{
	uint8_t config_register;
	
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Hier nach wird das config register ausgelesen.
	ds1620_write(0xAC, 8);
	config_register = ds1620_read(8);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
	
	return config_register;
}

//============================================================================//

uint16_t ds1620_read_thigh(void)
{
	uint16_t t_high;
	
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Hier nach wird das t-high register ausgelesen.
	ds1620_write(0xA1, 8);
	t_high = ds1620_read(9);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
	
	return t_high;
}

//============================================================================//

uint16_t ds1620_read_tlow(void)
{
	uint16_t t_low;
	
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Hier nach wird das t-low register ausgelesen.
	ds1620_write(0xA2, 8);
	t_low = ds1620_read(9);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
	
	return t_low;
}

//============================================================================//

uint16_t ds1620_read_temp(void)
{
	uint16_t temperature;
	
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Hier nach wird das temperatur register ausgelesen.
	ds1620_write(0xAA, 8);
	
	temperature = ds1620_read(9);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
	
	return temperature;
}

//============================================================================//

void ds1620_start(void)
{
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Starte den konvertierungs modus.
	ds1620_write(0xEE, 8);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
}

//============================================================================//

void ds1620_stop(void)
{
	// Reset = 1; Übertragung des befehls beginnt.
	DS1620_PORT |= (1 << DS1620_RST);
	
	// Beende den konvertierungs modus.
	ds1620_write(0x22, 9);
	
	// Reset = 0; Übertragung des befehls endet.
	DS1620_PORT &= ~(1 << DS1620_RST);
}

//============================================================================//

float ds1620_convert_temp(uint16_t temperature)
{
	if((temperature & 0x0100) == 1)
	{
		return (float)(temperature / 2.0f);
	}
	else
	{
		temperature &= 0x00FF;
		temperature = (0xFF - temperature) + 1;
		return -(float)(temperature / 2.0f);
	}
}

//============================================================================//

#endif // DS1620_H
